﻿#region GPL License

//Mediaportal (http://www.team-mediaportal.com/) Plugin - CentralizedDatabase
//Copyright (C) 2015 Max Wimmelbacher
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#endregion

using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Collections.ObjectModel;
using MediaPortal.GUI.Library;
using MediaPortal.Dialogs;
using MediaPortal.Player;
using MediaPortal.Video.Database;
using MediaPortal.Configuration;
using MediaPortal.Profile;
using MediaPortal.Plugins.MovingPictures;
using MediaPortal.Plugins.MovingPictures.MainUI;
using MediaPortal.Plugins.MovingPictures.Database;
using WindowPlugins.GUITVSeries;
using CentralizedDatabase.Configuration;
using System.Collections.Concurrent;
using Cornerstone.Database;
using Cornerstone.Database.Tables;
using Cornerstone.Database.CustomTypes;


namespace CentralizedDatabase
{
    [PluginIcons("CentralizedDatabase.Resources.CentralizedDatabaseEnabled.png", "CentralizedDatabase.Resources.CentralizedDatabaseDisabled.png")]

    public class CentralizedDatabase : GUIWindow, ISetupForm
    {

        #region Enumerations

        /// <summary>
        /// Enumeration of media types which are used to determine the correct databases to be updated.
        /// </summary>

        public enum MediaType
        {
            MPTVSeries,
            MPVideo,
            MovingPictures,
            Unknown
        };

        public enum EventType
        {
            StopOrChanged,
            Ended,
            Toggle,
            MP_UserSettingsUpdate,
            MP_WatchHistoryInsert,
            Unknown
        };

        #endregion

        #region Declaration

        /// <summary>
        /// Instantiate attributes on object construction.
        /// mCurrentMediaType is set to "Unknown" because on plugin startup no playback should be running.
        /// </summary>

        private CentralizedDatabaseConfiguration mConfiguration;
        private Collection<AdditionalVideoDatabase> mVideoDatabases = new Collection<AdditionalVideoDatabase>();
        private Collection<AdditionalTVSeriesDatabase> mTVSeriesDatabases = new Collection<AdditionalTVSeriesDatabase>();
        private Collection<AdditionalMovingPicturesDatabase> mMovingPicturesDatabases = new Collection<AdditionalMovingPicturesDatabase>();
        private static int mWatchedPercentageVideoDatabase;
        private static bool mMarkWatchedFilesVideoDatabase;
        private static int mWatchedPercentageTVSeriesDatabase;
        private CentralizedDatabase.MediaType mCurrentMediaType = MediaType.Unknown;
        static ConcurrentQueue<WatchedStateEvent> WatchStateEventQueue = new ConcurrentQueue<WatchedStateEvent>();
        // Create the thread object. This does not start the thread.
        WorkerThread workerObject = new WorkerThread();
        Thread workerThread = null;

        // WatchedStateEvent will be used to trigger the workerthread that it should do something
        public class WatchedStateEvent
        {
            public EventType EventType;
            public DateTime dtNextCheck;
            public object objDB;
            public MediaType MediaType;
            public string strFileName;
            public int intWatched;
            public int intResumeTime;
            //Used for gPlayer Event or Moving Pictures userSettings update
            public byte[] bteResumeData;
            public string strCurrentFile;
            public int intResume_BDTitle;
            public double dDuration;
            public double dCurrentPosition;
            public bool bolIsDVDMenu;
            //Used for Moving Pictures
            public int? intnullID;
            public int? intnullUserID;
            public int? intnullUserRating;
            public int? intnullMovieID;
            public DateTime dtDateWatched;
            public int intResumePart;
            //When bolUse_gPlayer is true, gPlayer data will be added automatically when event is fired
            public bool bolUse_gPlayer;

            //Initializer -> set default values
            public WatchedStateEvent()
            {

                EventType = EventType.Unknown;
                dtNextCheck = DateTime.Now;
                objDB = null;
                MediaType = MediaType.Unknown;
                strFileName = "";
                intWatched = 0;
                intResumeTime = 0;
                strCurrentFile = "";
                intResume_BDTitle = 0;
                dDuration = 0;
                dCurrentPosition = 0;
                bolIsDVDMenu = false;
                intnullID = 0;
                intnullUserID = 0;
                intnullUserRating = 0;
                intnullMovieID = 0;
                dtDateWatched = DateTime.Now;
                intResumePart = 0;
                bolUse_gPlayer = false;
                bteResumeData = null;
            }
        };

        #endregion

        #region ISetupForm Members

        /// <summary>
        /// This region is providing the necessary attributes/methods required by Mediaportal for the plugin configuration.
        /// </summary>

        /// <summary>
        /// Returns the Author of the Plugin to Mediaportal
        /// </summary>
        /// <returns>The Author of the Plugin</returns>
        public string Author()
        {
            return "Max Wimmelbacher, pOpY";
        }

        /// <summary>
        /// Boolean that decides whether the plugin can be enabled or not
        /// </summary>
        /// <returns>The boolean answer</returns>
        public bool CanEnable()
        {
            return true;
        }

        /// <summary>
        /// Decides if the plugin is enabled by default
        /// </summary>
        /// <returns>The boolean answer</returns>
        public bool DefaultEnabled()
        {
            return true;
        }

        /// <summary>
        /// Description of the plugin
        /// </summary>
        /// <returns>The Description</returns>
        public string Description()
        {
            return "Process plugin to update the watched information for media across several databases.";
        }

        /// <summary>
        /// Returns the items for the plugin
        /// </summary>
        /// <param name="strButtonText">The Buttons Text</param>
        /// <param name="strButtonImage">The Buttons Image</param>
        /// <param name="strButtonImageFocus">The Buttons Focused Image</param>
        /// <param name="strPictureImage">The Picture Image</param>
        /// <returns></returns>
        public bool GetHome(out string strButtonText, out string strButtonImage, out string strButtonImageFocus, out string strPictureImage)
        {
            strButtonText = string.Empty;
            strButtonImage = string.Empty;
            strButtonImageFocus = string.Empty;
            strPictureImage = string.Empty;
            return false;
        }

        /// <summary>
        /// Gets the Window id accociated with the plugin
        /// </summary>
        /// <returns>The window id</returns>
        public int GetWindowId()
        {
            //will be needed that MediaPortal calls the DeInit() function on stopping
            //TODO: which id should we use here??
            return 5678;
        }

        /// <summary>
        /// Boolean asking if the plugin has a setup setting
        /// </summary>
        /// <returns>The Boolean answer</returns>
        public bool HasSetup()
        {
            return true;
        }

        /// <summary>
        /// The Name of the Plugin
        /// </summary>
        /// <returns>The Name of the Plugin</returns>
        public string PluginName()
        {
            return "CentralizedDatabase";
        }

        /// <summary>
        /// Shows the Plugins configuration window
        /// </summary>
        public void ShowPlugin()
        {
            var Config = new CentralizedDatabaseConfig();
            Config.Show();
        }

        #endregion

        #region GUIWindow Overrides

        /// <summary>
        /// Needed that MediaPortal calls the Init() function on startup
        /// </summary>

        public override int GetID
        {
            get
            {
                //TODO: which id should we use here??
                return 5678;
            }
        }

        #endregion

        #region Init & DeInit function

        /// <summary>
        /// This region is providing the necessary methods required by Mediaportal for interaction with the plugin.
        /// Configuration Loading and hooking on the events necessary to trigger plugin actions is done here.
        /// </summary>

        /// <summary>
        /// Starting Point
        /// </summary>
        /// 
        public override bool Init()
        {
            int intDBCount;

            Log.Info("CentralizedDatabase: Starting...");
            intDBCount = LoadConfiguration();

            //Attach to the needed events
            AttachEvents();

            //Do we have any DBs confgiured and enabled?
            if (intDBCount > 0)
            {

                Log.Info("CentralizedDatabase: %i databases configured and enabled -> starting worker thread!");

                //Create the workerthread object
                workerThread = new Thread(workerObject.DoWork);

                // Start the worker thread.
                workerThread.Start();
            }
            else
            {
                //No databases configured or enabled -> dont start workerthread
                Log.Info("CentralizedDatabase: no databases configured and enabled -> we dont need an worker thread!");

                //Dettach to the needed events
                DettachEvents();

            }

            //Init OK
            return true;
        }

        /// <summary>
        /// End Point (Clean up)
        /// </summary>
        public override void DeInit()
        {
            Log.Info("CentralizedDatabase: Stopping");


            if (workerThread != null)
            {
                //We have started the worker thread and attached all events

                //Dettach to the needed events
                DettachEvents();

                // Request that the worker thread stop itself:
                workerObject.RequestStop();

                // Use the Join method to block the current thread  
                // until the object's thread terminates.
                workerThread.Join();
            }
            else
            {
                //No worker thread started and no attached events
                Log.Info("CentralizedDatabase: no running workerthread to stop!");
            }


            Log.Info("CentralizedDatabase: Stopped!");
            base.DeInit();
        }

        #endregion

        #region Event Registration / Unregistration

        /// <summary>
        /// Methods for attach/dettach to the needed Mediaportal/Plugins events
        /// </summary>

        void AttachEvents()
        {
            GUIWindowManager.Receivers += new SendMessageHandler(OnGUIMessageReceived);
            TVSeriesPlugin.ToggleWatched += new TVSeriesPlugin.ToggleWatchedEventDelegate(OnTVSeriesToggledWatched);
            MovingPicturesCore.DatabaseManager.ObjectUpdatedEx += new DatabaseManager.ObjectUpdatedDelegate(OnMovingPicturesDatabaseUpdated);
            MovingPicturesCore.DatabaseManager.ObjectInserted += new DatabaseManager.ObjectAffectedDelegate(OnMovingPicturesDatabaseInserted);
            g_Player.PlayBackEnded += OnPlayBackEnded;
            g_Player.PlayBackChanged += OnPlayBackChanged;
            g_Player.PlayBackStopped += OnPlayBackStopped;
        }

        void DettachEvents()
        {
            GUIWindowManager.Receivers -= new SendMessageHandler(OnGUIMessageReceived);
            TVSeriesPlugin.ToggleWatched -= new TVSeriesPlugin.ToggleWatchedEventDelegate(OnTVSeriesToggledWatched);
            MovingPicturesCore.DatabaseManager.ObjectUpdatedEx -= new DatabaseManager.ObjectUpdatedDelegate(OnMovingPicturesDatabaseUpdated);
            MovingPicturesCore.DatabaseManager.ObjectInserted -= new DatabaseManager.ObjectAffectedDelegate(OnMovingPicturesDatabaseInserted);
            g_Player.PlayBackEnded -= OnPlayBackEnded;
            g_Player.PlayBackChanged -= OnPlayBackChanged;
            g_Player.PlayBackStopped -= OnPlayBackStopped;
        }

        #endregion

        #region Mediaportal.GUI.Library.GUIWindowManager events

        /// <summary>
        /// The method OnGUIMessageReceived is listening for messages indicating that a media playback started and by whom.
        /// This information is used to determine which database needs to be updated after the media playback stopped.
        /// </summary>

        private void OnGUIMessageReceived(GUIMessage objMessage)
        {
            if (objMessage.Message == GUIMessage.MessageType.GUI_MSG_PLAYBACK_STARTED)
            {
                Log.Debug("CentralizedDatabase: Encountered message 'GUI_MSG_PLAYBACK_STARTED' for WindowId '" + GUIWindowManager.GetPreviousActiveWindow() + "'.");
                switch (GUIWindowManager.GetPreviousActiveWindow())
                {
                    case 25:
                    case 2003:
                        mCurrentMediaType = MediaType.MPVideo;
                        break;

                    case 2005:
                        //MovingPictures, set mCurrentMediaType to MediaType.Unknown so the user dont gets an warning and all gPlayer events will be ignored because MovingPictures works with events on function OnMovingPicturesDatabaseUpdated & OnMovingPicturesDatabaseInserted
                        mCurrentMediaType = MediaType.Unknown;
                        break;


                    case 9811:
                        mCurrentMediaType = MediaType.MPTVSeries;
                        break;
                    default:
                        {
                            mCurrentMediaType = MediaType.Unknown;
                            Log.Warn("CentralizedDatabase: Encountered WindowId '" + GUIWindowManager.GetPreviousActiveWindow() + "' which has no mediatype assigned.");
                            break;
                        }
                }
            }
        }

        #endregion

        #region MediaPortal.Player.g_player events

        /// <summary>
        /// This region defindes the methods to be run on triggering of hooked events.
        /// Based on the object which started the media playback it is decided which update method needs to be called.
        /// </summary>

        private void OnPlayBackEnded(g_Player.MediaType enuType, string strFileName)
        {
            WatchedStateEvent curEvent = new WatchedStateEvent();

            if (enuType == g_Player.MediaType.Video)
            {
                Log.Debug("CentralizedDatabase: 'g_Player.PlayBackEnded' event with media type '" + mCurrentMediaType + "' occured for file '" + strFileName + "'.");

                //Add nedded Watched Event data
                curEvent.EventType = EventType.StopOrChanged;
                curEvent.MediaType = mCurrentMediaType;
                curEvent.strFileName = strFileName;
                curEvent.intResumeTime = 0;
                curEvent.intWatched = 0;
                curEvent.bolUse_gPlayer = true;

                //Enqueue Event
                FireWatchedEvent(curEvent);
            }
        }

        private void OnPlayBackChanged(g_Player.MediaType enuType, int intStopTime, string strFileName)
        {
            WatchedStateEvent curEvent = new WatchedStateEvent();

            if (enuType == g_Player.MediaType.Video)
            {
                Log.Debug("CentralizedDatabase: 'g_Player.PlayBackChanged' event with media type '" + mCurrentMediaType + "' occured for file '" + strFileName + "'.");

                //Add nedded Watched Event data
                curEvent.EventType = EventType.StopOrChanged;
                curEvent.MediaType = mCurrentMediaType;
                curEvent.strFileName = strFileName;
                curEvent.intResumeTime = intStopTime;
                curEvent.intWatched = 0;
                curEvent.bolUse_gPlayer = true;

                //Enqueue Event
                FireWatchedEvent(curEvent);
            }
        }

        private void OnPlayBackStopped(g_Player.MediaType enuType, int intStopTime, string strFileName)
        {
            WatchedStateEvent curEvent = new WatchedStateEvent();

            if (enuType == g_Player.MediaType.Video)
            {
                Log.Debug("CentralizedDatabase: 'g_Player.PlayBackStopped' event with media type '" + mCurrentMediaType + "' occured for file '" + strFileName + "'.");

                //Add nedded Watched Event data
                curEvent.EventType = EventType.StopOrChanged;
                curEvent.MediaType = mCurrentMediaType;
                curEvent.strFileName = strFileName;
                curEvent.intResumeTime = intStopTime;
                curEvent.intWatched = 0;
                curEvent.bolUse_gPlayer = true;

                //Enqueue Event
                FireWatchedEvent(curEvent);
            }
        }

        #endregion

        #region WindowPlugins.GUITVSeries.TVSeriesPlugin events

        /// <summary>
        /// The method OnTVSeriesToggledWatched is listening for messages indicating that a episode was toggled watched using the menu in MediaPortal.
        /// This information is used to set the information related to Watched
        /// </summary>
        /// 

        private void OnTVSeriesToggledWatched(DBSeries objSeries, List<DBEpisode> lstDBEpisodes, bool bolWatched)
        {
            WatchedStateEvent curEvent = new WatchedStateEvent();

            foreach (DBEpisode objLoop in lstDBEpisodes)
            {
                Log.Debug("CentralizedDatabase: OnTVSeriesToggledWatched: The Watched Status for file '" + objLoop[DBEpisode.cFilename] + "' was toggled to '" + bolWatched + "',");

                //Add nedded Watched Event data
                curEvent.EventType = EventType.Toggle;
                curEvent.MediaType = MediaType.MPTVSeries;
                curEvent.strFileName = objLoop[DBEpisode.cFilename];
                curEvent.intResumeTime = 0;
                curEvent.intWatched = Convert.ToInt32(bolWatched);
                curEvent.bolUse_gPlayer = false;

                //Enqueue Event
                FireWatchedEvent(curEvent);
            }
        }

        #endregion

        #region MediaPortal.Plugins.MovingPictures events

        /// <summary>
        /// This region defindes the methods which will be called when MovingPictures updates or insert something in its database.
        /// Its used to get all the information needed for syncing it to the remote db.
        /// </summary>

        //OnMovingPicturesDatabaseUpdated, will be fired when an MovingPictures database object is updated
        private void OnMovingPicturesDatabaseUpdated(DatabaseTable dbObject, TableUpdateInfo ui)
        {
            WatchedStateEvent curEvent = new WatchedStateEvent();
            byte[] bteTmpResumeData = new byte[] { };

            // If it is user settings for a movie
            if (dbObject.GetType() != typeof(DBUserMovieSettings))
            {
                Log.Debug("CentralizedDatabase: OnMovingPicturesDatabaseUpdated: unknown object type: '" + dbObject.GetType().ToString() + "', event dropped!");
                return;
            }

            //Cast to DBUserMovieSettings object and get movie details
            DBUserMovieSettings userMovieSettings = (DBUserMovieSettings)dbObject;
            DBMovieInfo movie = userMovieSettings.AttachedMovies[0];

            //check if userMovieSettings.ResumeData is null or empty string
            //if that is the case reinitialize userMovieSettings.ResumeData with an empty byte array to prevent null exception when accessing userMovieSettings.ResumeData.Data
            //this should be fixed in ByteArray.cs of Cornerstone. There is access to _data.length and NO null check before!
            if ((userMovieSettings.ResumeData == null) || (userMovieSettings.ResumeData.ToString() == ""))
            {
                userMovieSettings.ResumeData = new ByteArray(bteTmpResumeData);
            }

            //Debug
            Log.Debug("CentralizedDatabase: OnMovingPicturesDatabaseUpdated: received DBUserMovieSettings data for file '" + movie.LocalMedia[0].FullPath + "':" +
                                                                "ID: '" + userMovieSettings.ID.ToString() + "', " +
                                                                "UserID: '" + userMovieSettings.User.ID.ToString() + "', " +
                                                                "UserRating: '" + userMovieSettings.UserRating.ToString() + "', " +
                                                                "WatchedCount: '" + userMovieSettings.WatchedCount.ToString() + "', " +
                                                                "ResumePart: '" + userMovieSettings.ResumePart.ToString() + "', " +
                                                                "ResumeTime: '" + userMovieSettings.ResumeTime.ToString() + "', " +
                                                                "ResumeTitleBD: '" + userMovieSettings.ResumeTitleBD.ToString() + "', " +
                                                                "bteResumeData: '" + userMovieSettings.ResumeData.ToString() + "'"
                                                                );
            //Add nedded Watched Event data
            curEvent.EventType = EventType.MP_UserSettingsUpdate;
            curEvent.MediaType = MediaType.MovingPictures;
            curEvent.strFileName = movie.LocalMedia[0].FullPath;
            curEvent.intWatched = userMovieSettings.WatchedCount;
            curEvent.intResumeTime = userMovieSettings.ResumeTime;
            curEvent.bteResumeData = userMovieSettings.ResumeData.Data;
            curEvent.intResume_BDTitle = userMovieSettings.ResumeTitleBD;
            curEvent.intnullID = userMovieSettings.ID;
            curEvent.intnullUserID = userMovieSettings.User.ID; ;
            curEvent.intnullUserRating = userMovieSettings.UserRating;
            curEvent.intResumePart = userMovieSettings.ResumePart;
            //Dont automatically add the gPlayer data in the function FireWatchedEvent -> we have all data here that we need -> set the data manually
            curEvent.bolUse_gPlayer = false;

            //Enqueue Event
            FireWatchedEvent(curEvent);
        }

        //OnMovingPicturesDatabaseInserted, will be fired when an MovingPictures database object is inserted
        private void OnMovingPicturesDatabaseInserted(DatabaseTable dbObject)
        {
            WatchedStateEvent curEvent = new WatchedStateEvent();

            // If it is user settings for a movie
            if (dbObject.GetType() != typeof(DBWatchedHistory))
            {
                Log.Debug("CentralizedDatabase: OnMovingPicturesDatabaseInserted: dbObject.GetType != DBWatchedHistory, event dropped!");
                return;
            }

            //Cast to DBWatchedHistory object and get movie details
            DBWatchedHistory watchedHistory = (DBWatchedHistory)dbObject;

            //Debug
            Log.Debug("CentralizedDatabase: OnMovingPicturesDatabaseInserted: received DBWatchedHistory data for file '" + watchedHistory.Movie.LocalMedia[0].FullPath + "':" +
                                                                "ID: '" + watchedHistory.ID.ToString() + "', " +
                                                                "UserID: '" + watchedHistory.User.ID.ToString() + "', " +
                                                                "MovieID: '" + watchedHistory.Movie.ID.ToString() + "', " +
                                                                "DateWatched UTC: '" + TimeZone.CurrentTimeZone.ToUniversalTime(watchedHistory.DateWatched).ToString() + "'"
                                                                );

            //Add nedded Watched Event data
            curEvent.EventType = EventType.MP_WatchHistoryInsert;
            curEvent.MediaType = MediaType.MovingPictures;
            curEvent.strFileName = watchedHistory.Movie.LocalMedia[0].FullPath;
            curEvent.intnullID = watchedHistory.ID;
            curEvent.intnullUserID = watchedHistory.User.ID;
            curEvent.intnullMovieID = watchedHistory.Movie.ID;
            curEvent.dtDateWatched = watchedHistory.DateWatched;
            //We dont need any gPlayer Data
            curEvent.bolUse_gPlayer = false;

            //Enqueue Event
            FireWatchedEvent(curEvent);
        }


        #endregion

        #region WorkerThread

        /// <summary>
        /// In this region the WorkerThread functions will be defined
        /// It consumes the Queued WatchedStateEvents and doesnt interrupt the GUI.
        /// </summary>

        public class WorkerThread
        {
            // This method will be called when the thread is started. 
            public void DoWork()
            {
                WatchedStateEvent evt;
                int iSleepMs = 10, iEventCount = 0, res;
                TimeSpan span;

                Log.Debug("CentralizedDatabase: WorkerThread started!");
                while (!_shouldStop)
                {
                    //SleepTime when there are no events

                    while (WatchStateEventQueue.TryDequeue(out evt))
                    {
                        iEventCount++;
                        if (evt.dtNextCheck <= DateTime.Now)
                        {
                            //we should process this item now
                            Log.Debug("CentralizedDatabase: WorkerThread: New event to process: " +
                                        "EventType: '" + evt.EventType.ToString() + "' - " +
                                        "dtNextCheck: '" + evt.dtNextCheck.ToString("HH:mm:ss.fff tt") + "' - " +
                                        "MediaType: '" + evt.MediaType.ToString() + "' - " +
                                        "strFileName: '" + evt.strFileName + "' - " +
                                        "intWatched: '" + evt.intWatched.ToString() + "' - " +
                                        "intResumeTime: '" + evt.intResumeTime.ToString() + "' - " +
                                        "strCurrentFile: '" + evt.strCurrentFile + "' - " +
                                        "intResume_BDTitle: '" + evt.intResume_BDTitle.ToString() + "' - " +
                                        "dDuration: '" + evt.dDuration.ToString() + "' - " +
                                        "dCurrentPosition: '" + evt.dCurrentPosition.ToString() + "' - " +
                                        "bolIsDVDMenu: '" + evt.bolIsDVDMenu.ToString() + "' - " +
                                        "intID: '" + evt.intnullID.ToString() + "' - " +
                                        "intUserID: '" + evt.intnullUserID.ToString() + "' - " +
                                        "intUserRating: '" + evt.intnullUserRating.ToString() + "' - " +
                                        "intnullMovieID: '" + evt.intnullMovieID.ToString() + "' - " +
                                        "dtDateWatched: '" + evt.dtDateWatched.ToString() + "' - " +
                                        "intResumePart: '" + evt.intResumePart.ToString() + "' - " +
                                        "bteResumeData: '" + ByteArrayToString(evt.bteResumeData) + "'"
                                        );


                            //Process the watchedevent
                            res = 0;
                            switch (evt.MediaType)
                            {
                                case MediaType.MPTVSeries:
                                    res = ProcessMPTVSeriesWatchedStateEvent(evt);
                                    break;

                                case MediaType.MovingPictures:
                                    res = ProcessMovingPicturesWatchedStateEvent(evt);
                                    break;

                                case MediaType.MPVideo:
                                    res = ProcessMPVideoWatchedStateEvent(evt);
                                    break;

                                default:
                                    Log.Debug("CentralizedDatabase: WorkerThread: dropped event with unknown MediaType!");
                                    break;


                            }

                            //check result and reschedule/retry if needed
                            if (res < 0)
                            {
                                //We should reschedule/retray the event
                                evt.dtNextCheck = DateTime.Now.AddSeconds(5);   //Retry in 5 Seconds
                                WatchStateEventQueue.Enqueue(evt);  //Event should be resheduled
                            }

                        }
                        else
                        {
                            //Event should be resheduled, readd it to queue
                            WatchStateEventQueue.Enqueue(evt);
                        }

                        //calculate complete millisconds to sleep
                        if (evt.dtNextCheck > DateTime.Now)
                        {
                            span = evt.dtNextCheck - DateTime.Now;
                            iSleepMs = (int)span.TotalMilliseconds;
                        }
                    }

                    //Minimal sleep is 10ms to give other proceses cpu time
                    if (iEventCount == 0) iSleepMs = 100;       //SleepTime when there are no events
                    if (iSleepMs < 10) iSleepMs = 10;
                    Thread.Sleep(iSleepMs);
                }
                Log.Debug("CentralizedDatabase: WorkerThread: terminating gracefully.");
            }
            public void RequestStop()
            {
                _shouldStop = true;
            }
            // Volatile is used as hint to the compiler that this data 
            // member will be accessed by multiple threads. 
            private volatile bool _shouldStop;
        }

        #endregion

        #region Private methods

        /// <summary>
        /// The method LoadConfiguration is called on plugin start and is deserializing the saved configuration.
        /// Database objects are created based on the configuration and stored in collections for the process lifetime.
        /// </summary>

        private int LoadConfiguration()
        {

            //Loading the plugins configuration.
            string strLocalDatabasePath = Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData) + @"\Team MediaPortal\MediaPortal\database\";
            bool bolCopiedMPTVSeries = false;
            bool bolCopiedVideos = false;
            bool bolCopiedMovingPictures = false;
            int intDBCount = 0;

            MPSettings objSettings = MPSettings.Instance;
            string strConfiguration = objSettings.GetValueAsString("CentralizedDatabase", "configuration", string.Empty);
            if (string.IsNullOrEmpty(strConfiguration))
            {
                mConfiguration.mConfigurationEntries = new List<CentralizedDatabaseConfigurationEntry>();
            }
            else
            {
                mConfiguration = Utils.Deserialize(strConfiguration);
            }

            //Loading the configured values when a media item is considered watched.

            mWatchedPercentageVideoDatabase = objSettings.GetValueAsInt("movies", "playedpercentagewatched", 95);
            mMarkWatchedFilesVideoDatabase = objSettings.GetValueAsBool("movies", "markwatched", true);
            mWatchedPercentageTVSeriesDatabase = (int)DBOption.GetOptions(DBOption.cWatchedAfter);

            //Create object collections containing all configured databases which needs to be updated.

            foreach (CentralizedDatabaseConfigurationEntry objLoop in mConfiguration.mConfigurationEntries)
            {
                if (objLoop.mEnabled == true)
                {
                    intDBCount++;
                    switch (objLoop.mDatabaseType)
                    {
                        case "Video database":
                            AdditionalVideoDatabase objVideoDatabase = new AdditionalVideoDatabase(objLoop.mDatabasePath);
                            objVideoDatabase.Dispose();
                            mVideoDatabases.Add(objVideoDatabase);
                            //Copy DB to Client?
                            if (!bolCopiedVideos && objLoop.mCopyAtStartup)
                            {
                                //CopyDB
                                CopyDB(objLoop.mDatabasePath, strLocalDatabasePath + Path.GetFileName(objLoop.mDatabasePath));
                            }
                            break;
                        case "TVSeries database":
                            AdditionalTVSeriesDatabase objTVSeriesDatabase = new AdditionalTVSeriesDatabase(objLoop.mDatabasePath);
                            objTVSeriesDatabase.Dispose();
                            mTVSeriesDatabases.Add(objTVSeriesDatabase);
                            //Copy DB to Client?
                            if (!bolCopiedMPTVSeries && objLoop.mCopyAtStartup)
                            {
                                //CopyDB
                                CopyDB(objLoop.mDatabasePath, strLocalDatabasePath + Path.GetFileName(objLoop.mDatabasePath));
                            }
                            break;
                        case "MovingPictures database":
                            AdditionalMovingPicturesDatabase objMovingPicturesDatabase = new AdditionalMovingPicturesDatabase(objLoop.mDatabasePath);
                            objMovingPicturesDatabase.Dispose();
                            mMovingPicturesDatabases.Add(objMovingPicturesDatabase);
                            //Copy DB to Client?
                            if (!bolCopiedMovingPictures && objLoop.mCopyAtStartup)
                            {
                                //CopyDB
                                CopyDB(objLoop.mDatabasePath, strLocalDatabasePath + Path.GetFileName(objLoop.mDatabasePath));
                            }
                            break;
                    }
                }
            }
            return intDBCount;
        }

        //Method CopyDB copies a remote db to the local database path
        private void CopyDB(string strSourceFile, string strDestFile)
        {
            //CopyDB
            try
            {
                File.Copy(strSourceFile, strDestFile, true);
                Log.Info("CentralizedDatabase: LoadConfiguration: database: '{0}' copied to '{1}'", strSourceFile, strDestFile);
            }
            catch (Exception ex)
            {
                Log.Error("CentralizedDatabase: LoadConfiguration: exception on copy database: '{0}' to '{1}, ex:{2} stack:{3}", strSourceFile, strDestFile, ex.Message, ex.StackTrace);
            }
        }

        //ByteArrayToString, helper function to convert byte array to a readable format
        public static string ByteArrayToString(byte[] ba)
        {
            if (ba == null) return "";
            StringBuilder hex = new StringBuilder(ba.Length * 3);
            foreach (byte b in ba)
                hex.AppendFormat("{0:x2} ", b);
            return hex.ToString();
        }

        //Method FireWatchedEvent will be called on any Mediaportal Stop, change, toggle, db update or insert event and adds the needed data to the WatchStateEventQueue
        private void FireWatchedEvent(WatchedStateEvent evt)
        {
            //Add needed g_Player data if evt.bolUse_gPlayer is set
            if (evt.bolUse_gPlayer)
            {
                try
                {
                    evt.intResume_BDTitle = g_Player.SetResumeBDTitleState;
                    evt.dCurrentPosition = g_Player.Player.CurrentPosition;
                    evt.strCurrentFile = g_Player.CurrentFile;
                    evt.dDuration = g_Player.Player.Duration;
                    evt.bolIsDVDMenu = g_Player.IsDVDMenu;
                    g_Player.Player.GetResumeState(out evt.bteResumeData);
                }
                catch (Exception ex)
                {
                    Log.Error("CentralizedDatabase: FireWatchedEvent:exception err:{0} stack:{1}", ex.Message, ex.StackTrace);
                }
            }

            //DateTime when the item should be processed
            evt.dtNextCheck = DateTime.Now;

            //Enqueue Event for every configured database
            switch (evt.MediaType)
            {
                case MediaType.MPTVSeries:
                    foreach (AdditionalTVSeriesDatabase objLoop in mTVSeriesDatabases)
                    {
                        evt.objDB = objLoop;
                        WatchStateEventQueue.Enqueue(evt);
                    }
                    break;

                case MediaType.MovingPictures:
                    foreach (AdditionalMovingPicturesDatabase objLoop in mMovingPicturesDatabases)
                    {
                        evt.objDB = objLoop;
                        WatchStateEventQueue.Enqueue(evt);
                    }
                    break;

                case MediaType.MPVideo:
                    foreach (AdditionalVideoDatabase objLoop in mVideoDatabases)
                    {
                        evt.objDB = objLoop;
                        WatchStateEventQueue.Enqueue(evt);
                    }
                    break;

                default:
                    Log.Debug("CentralizedDatabase: FireWatchedEvent: unknown MediaType. event dropped!");
                    break;


            }

        }

        #region Video databases

        /// <summary>
        /// Methods in this region are run if the media type is "MPVideo" and a gplayer event was triggered.
        /// The defined methods run the necessary commands to update all configured video databases.
        /// </summary>

        //Method ProcessMPVideoWatchedStateEvent is run when an event for videos databases should be processed
        //returns:
        //  1 = Infos written to given DB -> event could be cleared from queue
        //  0 = other error (ex. file not found in db, file does not exists...) -> event could be cleared from queue
        //  <0 = DB is locked by other user -> event should NOT be cleared from queue -> retry


        private static int ProcessMPVideoWatchedStateEvent(WatchedStateEvent evt)
        {
            bool bolStacked = false;
            AdditionalVideoDatabase MPVideoDB = (AdditionalVideoDatabase)evt.objDB;

            MPVideoDB.ReOpen();
            bool bolUpdated = false;
            ArrayList alsFiles = new ArrayList();
            int intDuration = 0;
            int intTotalDuration = 0;
            int intPlayTimePercentage = 0;

            //Get MovieID and all its correlated media file(s).

            int intMovieId = MPVideoDB.GetMovieId(evt.strFileName);
            if (intMovieId == -1)
            {
                Log.Warn("CentralizedDatabase: ProcessMPVideoWatchedStateEvent: File '" + evt.strFileName + "' is not available in database '" + MPVideoDB.DatabaseName + "'.");
                MPVideoDB.Dispose();
                return 0;
            }
            else
            {
                MPVideoDB.GetFilesForMovie(intMovieId, ref alsFiles);
            }

            //Check if MovieID consists of several media files.

            if (alsFiles.Count <= 0)
            {
                Log.Warn("CentralizedDatabase: ProcessMPVideoWatchedStateEvent: File '" + evt.strFileName + "' is not available in database '" + MPVideoDB.DatabaseName + "'.");
                MPVideoDB.Dispose();
                return 0;
            }
            else if (alsFiles.Count > 1)
            {
                bolStacked = true;
            }

            //Calculates the total duration of all media files of the MovieID.

            foreach (string strLoop in alsFiles)
            {
                intTotalDuration += MPVideoDB.GetVideoDuration(MPVideoDB.GetFileId(strLoop));
            }

            //do calculations depending on the EventType
            switch (evt.EventType)
            {

                case EventType.Ended:

                    //Resets the MovieStopTime and the ResumeData for the media file which was last played.

                    for (int intLoop = 0; intLoop < alsFiles.Count; intLoop++)
                    {
                        string strFilePath = (string)alsFiles[intLoop];
                        byte[] bteResumeData = null;
                        int intFileId = MPVideoDB.GetFileId(strFilePath);
                        if (intFileId < 0)
                        {
                            break;
                        }
                        MPVideoDB.GetMovieStopTimeAndResumeData(intFileId, out bteResumeData, evt.intResume_BDTitle);
                        MPVideoDB.SetMovieStopTimeAndResumeData(intFileId, 0, bteResumeData, evt.intResume_BDTitle);
                        bolUpdated = true;
                    }

                    //Calculates the Watched percentage of the movie based on all media items correlated.

                    if (bolStacked && intTotalDuration != 0)
                    {
                        for (int i = 0; i < alsFiles.Count; i++)
                        {
                            int intFileId = MPVideoDB.GetFileId((string)alsFiles[i]);
                            if (evt.strFileName != (string)alsFiles[i])
                            {
                                intDuration += MPVideoDB.GetVideoDuration(intFileId);
                                continue;
                            }
                            intPlayTimePercentage = (int)(100 * (intDuration + evt.dCurrentPosition) / intTotalDuration);
                            break;
                        }
                    }
                    else
                    {
                        intPlayTimePercentage = 100;
                    }

                    break;

                case EventType.StopOrChanged:

                    //Calculates the Watched percentage of the movie based on all media items correlated.
                    if (bolStacked && intTotalDuration != 0)
                    {
                        for (int intLoop = 0; intLoop < alsFiles.Count; intLoop++)
                        {
                            if (evt.strCurrentFile != (string)alsFiles[intLoop])
                            {
                                intDuration += MPVideoDB.GetVideoDuration(MPVideoDB.GetFileId((string)alsFiles[intLoop]));
                                continue;
                            }
                            intPlayTimePercentage = (100 * (intDuration + evt.intResumeTime) / intTotalDuration);
                            break;
                        }
                    }
                    else
                    {
                        if (evt.dDuration >= 1)
                        {
                            intPlayTimePercentage = (int)Math.Ceiling((evt.intResumeTime / evt.dDuration) * 100);
                        }
                    }

                    //Sets the Watched status and correlated for the media file which was last played.

                    for (int intLoop = 0; intLoop < alsFiles.Count; intLoop++)
                    {
                        string strFilePath = (string)alsFiles[intLoop];
                        int intFileId = MPVideoDB.GetFileId(strFilePath);
                        intMovieId = MPVideoDB.GetMovieId(strFilePath);

                        //Check if a DVD/Blueray was played and set the Watched status and correlated according to that.

                        if (evt.bolIsDVDMenu)
                        {
                            MPVideoDB.SetMovieStopTimeAndResumeData(intFileId, 0, null, evt.intResume_BDTitle);
                            MPVideoDB.SetMovieWatchedStatus(intMovieId, true, 100);
                            MPVideoDB.MovieWatchedCountIncrease(intMovieId);
                        }

                        //Check if the media file currently played is the one in the loop and set the Watched status and correlated according to that.

                        else if ((evt.strFileName.Trim().ToLowerInvariant().Equals(strFilePath.Trim().ToLowerInvariant())) && (evt.intResumeTime > 0))
                        {
                            MPVideoDB.GetMovieStopTimeAndResumeData(intFileId, out evt.bteResumeData, evt.intResume_BDTitle);
                            MPVideoDB.SetMovieStopTimeAndResumeData(intFileId, evt.intResumeTime, evt.bteResumeData, evt.intResume_BDTitle);

                            //Updates the Watched status for all media files based on the configuration done for the video database.

                            if (intPlayTimePercentage >= mWatchedPercentageVideoDatabase)
                            {
                                MPVideoDB.SetMovieWatchedStatus(intMovieId, true, intPlayTimePercentage);
                                MPVideoDB.MovieWatchedCountIncrease(intMovieId);
                                bolUpdated = true;
                            }
                            else
                            {
                                int intPercent = 0;
                                int intTimesWatched = 0;
                                bool bolWatched = MPVideoDB.GetMovieWatchedStatus(intMovieId, out intPercent, out intTimesWatched);
                                MPVideoDB.SetMovieWatchedStatus(intMovieId, bolWatched, intPlayTimePercentage);
                                bolUpdated = true;
                            }
                        }
                        else
                        {
                            MPVideoDB.DeleteMovieStopTime(intFileId);
                        }
                    }

                    break;

                case EventType.Toggle:
                default:
                    Log.Debug("CentralizedDatabase: ProcessMPVideoWatchedStateEvent: unkown EventType: " + evt.EventType.ToString());
                    break;

            }



            //Updates the Watched status based on the configuration done for the video database.
            if (mMarkWatchedFilesVideoDatabase)
            {
                IMDBMovie objIMDBMovie = new IMDBMovie();
                MPVideoDB.GetMovieInfoById(intMovieId, ref objIMDBMovie);
                if (!objIMDBMovie.IsEmpty)
                {

                    //Updates the Watched status for all media files based on the configuration done for the video database. 
                    if (intPlayTimePercentage >= mWatchedPercentageVideoDatabase)
                    {
                        objIMDBMovie.Watched = 1;

                        if (evt.EventType == EventType.Ended)
                        {
                            MPVideoDB.SetWatched(objIMDBMovie);
                            MPVideoDB.SetMovieWatchedStatus(intMovieId, true, intPlayTimePercentage);
                            MPVideoDB.MovieWatchedCountIncrease(intMovieId);
                            MPVideoDB.SetDateWatched(objIMDBMovie);
                        }
                        else if (evt.EventType == EventType.Ended)
                        {
                            objIMDBMovie.DateWatched = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                            MPVideoDB.SetMovieInfoById(intMovieId, ref objIMDBMovie);
                        }
                        bolUpdated = true;
                    }
                    else if (evt.EventType == EventType.Ended)
                    {
                        int intPercent = 0;
                        int intTimesWatched = 0;
                        bool bolWatched = MPVideoDB.GetMovieWatchedStatus(intMovieId, out intPercent, out intTimesWatched);
                        MPVideoDB.SetMovieWatchedStatus(intMovieId, bolWatched, intPlayTimePercentage);
                        bolUpdated = true;
                    }



                }
            }

            //Check if update was successfull and write to media portal log.

            if (bolUpdated == true)
            {
                Log.Info("CentralizedDatabase: ProcessMPVideoWatchedStateEvent: Information for file '" + evt.strFileName + "' was updated in database '" + MPVideoDB.DatabaseName + "'.");
            }
            MPVideoDB.Dispose();

            //OK, clear event from queue
            return 1;
        }



        #endregion


        #region TVSeries databases

        /// <summary>
        /// Methods in this region are run if the media type is "MPTVSeries" or the trigger was an event related to "MPTVSeries".
        /// The defined methods run the necessary commands to update all configured video databases.
        /// </summary>


        //Method ProcessMPTVSeriesWatchedStateEvent is run when an event for MPTVSeries databases should be processed
        //returns:
        //  1 = Infos written to given DB -> event could be cleared from queue
        //  0 = other error (ex. file not found in db, file does not exists...) -> event could be cleared from queue
        //  <0 = DB is locked by other user -> event should NOT be cleared from queue -> retry

        private static int ProcessMPTVSeriesWatchedStateEvent(WatchedStateEvent evt)
        {
            int intPlayTimePercentage = 0;
            AdditionalTVSeriesDatabase TVSeriesDB = (AdditionalTVSeriesDatabase)evt.objDB;

            TVSeriesDB.ReOpen();
            bool bolUpdated = false;
            ArrayList alsCompositeIds = new ArrayList();

            //Get CompositeId(s).

            string strCompositeId = TVSeriesDB.GetCompositeId(evt.strFileName);
            if (strCompositeId == string.Empty)
            {
                Log.Warn("CentralizedDatabase: File '" + evt.strFileName + "' is not available in database '" + TVSeriesDB.DatabaseName + "'.");
                TVSeriesDB.Dispose();
                return 0;
            }
            else
            {
                TVSeriesDB.GetCompositeIdsForEpisode(strCompositeId, ref alsCompositeIds);
            }
            if (alsCompositeIds.Count <= 0)
            {
                Log.Warn("CentralizedDatabase: File '" + evt.strFileName + "' is not available in database '" + TVSeriesDB.DatabaseName + "'.");
                TVSeriesDB.Dispose();
                return 0;
            }

            //Calculates the Watched percentage of the episode.

            if ((evt.dDuration >= 1) && (evt.EventType == EventType.StopOrChanged))
            {
                intPlayTimePercentage = (int)Math.Ceiling((evt.intResumeTime / evt.dDuration) * 100);
            }
            else if (evt.EventType == EventType.Ended)
            {
                intPlayTimePercentage = 100;
            }

            //Updates the Watched status for all CompositeIds.

            for (int intLoop = 0; intLoop < alsCompositeIds.Count; intLoop++)
            {
                strCompositeId = (string)alsCompositeIds[intLoop];
                bool bolWatched = false;
                string strWatchedDate;
                int intStopTime;

                //Check if episode was already watched.

                TVSeriesDB.GetEpisodeWatchedStatus(strCompositeId, out bolWatched, out strWatchedDate, out intStopTime);

                //Updates the Watched status based on the configuration done for the tvseries database

                if (((evt.EventType != EventType.Toggle) && (intPlayTimePercentage >= mWatchedPercentageTVSeriesDatabase)) || ((evt.EventType == EventType.Toggle) && (bolWatched == false && evt.intWatched == 1)))
                {
                    if (((evt.EventType != EventType.Toggle) && (bolWatched == false)) || ((evt.EventType == EventType.Toggle) && (bolWatched == false && evt.intWatched == 1)))
                    {

                        //Update the Watched status for the episodes and correlated it is unwatched.

                        TVSeriesDB.SetEpisodeWatchedStatus(strCompositeId, true, DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss"), 0, true);
                        if (evt.EventType == EventType.Toggle) TVSeriesDB.EpisodePlayCountIncrease(strCompositeId, true);
                        string strSeasonId;
                        bool bolSeasonWatched = false;
                        TVSeriesDB.GetSeasonWatchedStatus(strCompositeId, out strSeasonId, out bolSeasonWatched);
                        if (bolSeasonWatched == false)
                        {

                            //Update the Unwatched episodes count for the season if it is unwatched.

                            TVSeriesDB.SeasonUnWatchedCountDecrease(strSeasonId);
                            string strSeriesId;
                            bool bolSeriesWatched = false;
                            TVSeriesDB.GetSeriesWatchedStatus(strCompositeId, out strSeriesId, out bolSeriesWatched);
                            if (bolSeriesWatched == false)
                            {

                                //Update the Unwatched episodes count for the series if it is unwatched.

                                TVSeriesDB.SeriesUnWatchedCountDecrease(strSeriesId);
                            }
                        }
                    }
                    else
                    {
                        if (evt.EventType == EventType.StopOrChanged) strWatchedDate = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                        TVSeriesDB.SetEpisodeWatchedStatus(strCompositeId, bolWatched, strWatchedDate, 0);
                    }
                    TVSeriesDB.EpisodePlayCountIncrease(strCompositeId);
                    bolUpdated = true;
                }
                else
                {
                    if (evt.EventType == EventType.Ended) intStopTime = 0;
                    if (evt.EventType == EventType.StopOrChanged) intStopTime = evt.intResumeTime;
                    if (evt.EventType == EventType.Toggle) bolWatched = Convert.ToBoolean(evt.intWatched);

                    //Update Watched State
                    TVSeriesDB.SetEpisodeWatchedStatus(strCompositeId, bolWatched, strWatchedDate, intStopTime);

                    if (evt.EventType == EventType.Ended) TVSeriesDB.EpisodePlayCountIncrease(strCompositeId);
                    bolUpdated = true;
                }
            }

            //Check if update was successfull and write to media portal log.

            if (bolUpdated == true)
            {
                Log.Info("CentralizedDatabase: Information for file '" + evt.strFileName + "' was updated in database '" + TVSeriesDB.DatabaseName + "'.");
            }
            TVSeriesDB.Dispose();

            //OK, clear event from queue
            return 1;
        }

        #endregion

        #region MovingPictures databases

        /// <summary>
        /// Methods in this region are run if the media type is "MovingPictures" or the trigger was an event related to "MovingPictures".
        /// The defined methods run the necessary commands to update all configured video databases.
        /// </summary>

        //Method ProcessMovingPicturesWatchedStateEvent is run when an event for moving pictures databases should be processed
        //returns:
        //  1 = Infos written to given DB -> event could be cleared from queue
        //  0 = other error (ex. file not found in db, file does not exists...) -> event could be cleared from queue
        //  <0 = DB is locked by other user -> event should NOT be cleared from queue -> retry

        private static int ProcessMovingPicturesWatchedStateEvent(WatchedStateEvent evt)
        {
            int iRes = -255;
            AdditionalMovingPicturesDatabase MovingPicturesDB = (AdditionalMovingPicturesDatabase)evt.objDB;

            MovingPicturesDB.ReOpen();

            //known Moving Pictures Eventtype?
            switch (evt.EventType)
            {
                case EventType.MP_UserSettingsUpdate:
                    //Update DBUserMovieSettings table
                    iRes = MovingPicturesDB.UpdateDBUserMovieSettings(evt.intnullID, evt.intnullUserID, evt.intnullUserRating, evt.intWatched, evt.intResumePart, evt.intResumeTime, evt.bteResumeData, evt.intResume_BDTitle);
                    break;

                case EventType.MP_WatchHistoryInsert:
                    //Insert DBWatchedHistory table
                    iRes = MovingPicturesDB.InsertDBWatchedHistory(evt.intnullUserID, evt.intnullMovieID, evt.dtDateWatched);
                    break;

                default:
                    Log.Debug("CentralizedDatabase: ProcessMovingPicturesWatchedStateEvent: unkown eventtyp: " + evt.EventType.ToString());
                    break;
            }

            //Check if update was successfull and write to media portal log.

            if (iRes == 1)
            {
                Log.Info("CentralizedDatabase: Information for file '" + evt.strFileName + "' was updated in database '" + MovingPicturesDB.DatabaseName + "'.");
            }
            else
            {
                Log.Warn("CentralizedDatabase: Information for file '" + evt.strFileName + "' was NOT updated in database '" + MovingPicturesDB.DatabaseName + "'. iRes= " + iRes.ToString());
            }

            //Dispose Database
            MovingPicturesDB.Dispose();

            //OK, clear event from queue
            return 1;
        }

        #endregion

        #endregion


    }
}